#include "cmsis_os.h"
#include "stm32f2xx.h"
#include "LED.h"
#include "GLCD.h"
#include <stdio.h>
#include <stdint.h>

#include "lwip/dhcp.h"
#include "lwip/tcp_impl.h"
#include "lwip/init.h"
#include "lwip/netif.h"
#include "netif/etharp.h"
#include "ethernetif.h"
#include "httpd.h"
 
#define NET_TICK_INTERVAL_MS  10

static void net_init (void);
static void net_periodic (uint32_t localtime);
static void net_timer_tick (void const *arg);
osTimerDef (net_tick, net_timer_tick);

struct netif netif;
uint32_t LocalTime = 0;
uint8_t  LEDs = 0;


static void net_timer_tick (void const *arg) {
  /* System tick timer callback function */
  LocalTime += NET_TICK_INTERVAL_MS;
}


static void net_init (void) {
  struct ip_addr ipaddr;
  struct ip_addr netmask;
  struct ip_addr gw;
  osTimerId id;

  lwip_init ();

#if LWIP_DHCP
  ipaddr.addr = 0;
  netmask.addr = 0;
  gw.addr = 0;
#else
  IP4_ADDR(&ipaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
  IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1 , NETMASK_ADDR2, NETMASK_ADDR3);
  IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
#endif

  /* Add your network interface to the netif_list. */
  netif_add(&netif, &ipaddr, &netmask, &gw, NULL, &ethernetif_init, &ethernet_input);

  /*  Registers the default network interface.*/
  netif_set_default(&netif);

#if LWIP_DHCP
  dhcp_start (&netif);
#else
  netif_set_up(&netif);
#endif

  id = osTimerCreate (osTimer (net_tick), osTimerPeriodic, NULL);
  osTimerStart (id, NET_TICK_INTERVAL_MS);
}


static void net_periodic (uint32_t localtime) {
  static uint32_t TCPTimer = 0;
  static uint32_t ARPTimer = 0;
  static uint32_t LEDTimer = 0;
#if LWIP_DHCP
  static uint32_t DHCPfineTimer   = 0;
  static uint32_t DHCPcoarseTimer = 0;
#endif
  static uint32_t IPaddress = 1;
  static char buff[20];
  uint8_t ip[4];

#if LWIP_TCP
  /* TCP periodic process every 250 ms */
  if (localtime - TCPTimer >= TCP_TMR_INTERVAL) {
    TCPTimer = localtime;
    tcp_tmr();
  }
#endif
  
  /* ARP periodic process every 5s */
  if ((localtime - ARPTimer) >= ARP_TMR_INTERVAL) {
    ARPTimer = localtime;
    etharp_tmr();
  }

#if LWIP_DHCP
  /* Fine DHCP periodic process every 500ms */
  if (localtime - DHCPfineTimer >= DHCP_FINE_TIMER_MSECS) {
    DHCPfineTimer = localtime;
    dhcp_fine_tmr();
  }

  /* DHCP Coarse periodic process every 60s */
  if (localtime - DHCPcoarseTimer >= DHCP_COARSE_TIMER_MSECS) {
    DHCPcoarseTimer = localtime;
    dhcp_coarse_tmr();
  }
#endif

  /* Link check and LED blink process every 500ms */
  if ((localtime - LEDTimer) >= 500) {
    LEDTimer = localtime;
    ethernetif_check_link (&netif);
    if (netif_is_link_up(&netif)) {
      LEDs |= 0x80;
      /* Print IP address on LCD display */
      if (IPaddress != netif.ip_addr.addr) {
        IPaddress = netif.ip_addr.addr;
        ip[0] = (uint8_t)(IPaddress);
        ip[1] = (uint8_t)(IPaddress >> 8);
        ip[2] = (uint8_t)(IPaddress >> 16);
        ip[3] = (uint8_t)(IPaddress >> 24);

        GLCD_ClearLn (4,1);
        sprintf (buff, "IP: %d.%d.%d.%d",ip[0],ip[1],ip[2],ip[3]);
        GLCD_DisplayString (4, 0, 1, buff);
      }
    }
    else {
      LEDs &= ~0x80;
      if (IPaddress) {
        IPaddress = 0;
        GLCD_ClearLn (4, 1);
        GLCD_DisplayString (4, 0, 1, "Link Down...");
      }
    }
    LEDs ^= 0x01;
    LED_Out (LEDs);
  }
}


int main(void) {
  LED_Initialize ();
  GLCD_Initialize ();

  GLCD_Clear (Maroon);
  GLCD_SetBackColor (Maroon);
  GLCD_SetTextColor (White);

  GLCD_DisplayString (1, 4, 1, "RTE and LwIP");
  GLCD_DisplayString (2, 2, 1, "HTTP demo example");

  net_init ();
  httpd_init ();

  /* Infinite loop */
  while (1) {
    /* check if any packet received */
    ethernetif_poll (&netif);
    /* handle periodic timers for LwIP */
    net_periodic (LocalTime);
  }
}


